%%% Filename : p_axiom2rule.pl
%%% Date     : 28.03.2012
%%% Contents : main method to create a rule from an axiom
%%% Author   : Lara Katharina Spendier <lara@logic.at>


%% startComputation(+Axiom, +Number, +Star1, +Star2, +Predefined)
%%	creates the logical rules out of the axiom(s) provided by the user
%% 	Axiom 	... Axiom(s) provided by the user
%%	Number 		... 0 = LK+, 2 = BK (1 "one predefined connective; not an option any more)
%%	[Star1,Star2] 	... ID for the new unary connectives; 
%%				1 ... negation, 10 ... circle
%%	Predefined ... 1 if '*2' is invertible, 0 otherwise
startComputation(Axiom, N, S1, S2, P) :-
	writef('\n\nEquivalent Logical Rule(s): \n'),
	axioms2rules(Axiom, P, Logical, LogicalR),
	is_matrices(LogicalR, N, S1, S2, Matrices),
	encodingOut(Axiom, LogicalR, N),
	splitAxiom([Axiom], SAxiom),
	texOutAx(SAxiom, TAxiom),
	length(SAxiom, NumberAxioms),
	texOut(TAxiom, Logical, Matrices, N, NumberAxioms).

startComputation(0, 0) :-
	isTruthTable2(Val, TVal, And, Or, Implies, Star1, Star2),
	MatrixTV = [Val, TVal, And,Or,Implies,Star1,Star2], 
	nl, nl, print_matrixtv(MatrixTV).
startComputation(0, _) :-
	isTruthTable2(Val, TVal, And, Or, Implies, Star1, Star2),
	MatrixTV = [Val, TVal, And,Or,Implies,Star1,Star2], 
	isUnaryConnective(1, '*1', Rule1),
	isUnaryConnective(10, '*2', Rule2),
	computeVM(MatrixTV, Rule1, VM),
	MatrixTV1 = [Val, VM, And, Or, Implies, Star1, Star2],
	computeVM(MatrixTV1, Rule2, VM1),
	isUnaryTruthTable(VM1, '*1', Star11),
	computeLVM(Star11, [], '*1', Star12),
	isUnaryTruthTable(VM1, '*2', Star21),
	computeLVM(Star21, [], '*2', Star22),
	computeBLVM(And, VM1, [], '&', And1),
	computeBLVM(Or, VM1, [], 'v', Or1),
	computeBLVM(Implies, VM1, [], '->', Implies1),
	MTV = [Val, VM1, And1, Or1, Implies1, Star12, Star22],
	nl, nl, print_matrixtv(MTV).

%% axioms2rules(+Axiom, +Predefined, -Logical, -LogicalR)
%%	splits axioms that are concatenated with '&'
%% 	Axiom 	... Axiom provided by the user
%%	Predefined ... 1 if '*2' is invertible, 0 otherwise
%%	Logical ... Logical Rules created out of the axioms
%%	LogicalR ... logical Rules rewritten
axioms2rules([], _, [], []).
axioms2rules(X ';' Y, P, L, LR) :-
	axiom2rule(X, P, L1, LR1),
	axioms2rules(Y, P, L2, LR2),
	append(L1, L2, L),
	append(LR1, LR2, LR).
axioms2rules(X, P, L, LR) :-
	axiom2rule(X, P, L, LR).

%% axiom2rule(+Axiom, +Predefined, -Logical, -LogicalR)
%%	axiom to rule
%% 	Axiom 	... Axiom provided by the user
%%	Predefined ... 1 if '*2' is invertible, 0 otherwise
%%	Logical ... Logical Rules created out of the axioms
%%	LogicalNew ... logical Rules rewritten
axiom2rule(Axiom, Predefined, Logical, LogicalNew) :-
	is_rules([Axiom], Logical, Predefined),
	Logical = [L1],
	rewriteLogicalRules(L1, LogicalNew).

%% is_rules(+Axioms, -Logical)
%%	creates the logical rules out of the axiom provided by the user
%% 	Axioms   ... List of axioms (provided by the user)
%%	Logical  ... List of logical rules according to Axioms
%%	Predefined ... 1 if '*2' is invertible, 0 otherwise
is_rules([], [], _).
is_rules([H|T], Logical, Predefined) :-
	axiom2logical(H, Logical1, Predefined),
	printrule(Logical1),
	is_rules(T, Logical2, Predefined),
	append([Logical1], Logical2, Logical).

%% is_matrices(+Logical, +N, +Star1, +Star2, -Matrices)
%%	creates the matrices (truth tables) out of the Logical Rules and the Base Calculus
%%	Logical  ... List of logical rules according to Axioms
%%	Number 		... Number of new unary connectives 
%%			    1 ... 1 new unary connective, baseCalculus = 4 TV
%%			    2 ... 2 new unary connectives	
%%	[Star1,Star2] 	... ID for the new unary connectives; 
%%				1 ... negation, 10 ... circle
%%	Matrices ... List of matrices for the connectives

is_matrices(Logical, N, S1, S2, Matrices) :-
	is_matrices_base(N, S1, S2, Logical, MatrixBase, _),
	is_matrices_unary(MatrixBase, Logical, MatrixCompute),
	nl, nl, print_matrixtv(MatrixCompute),
	Matrices = MatrixCompute.

%% texOutAx(+SAxiom, -TAxiom)
%%	creates the texOutput for the list of axioms / logical rules
texOutAx([],'').
texOutAx([H|T], TAxiom) :-
	texOutAx(T, TAxiom1),
	axiom2tex(H, TAxiom2, []),
	(T = [] ->
	  TAxiom = TAxiom2
%	; append(TAxiom2, [92], TAxiom3),
%	  append(TAxiom3, "qquad", TAxiom4),
%	  append(TAxiom4, TAxiom1, TAxiom)
	; append(TAxiom2, ",", TAxiom3),
	  append(TAxiom3, TAxiom1, TAxiom)
	).


rewriteLogicalRules([], []).
rewriteLogicalRules([H|T], NewRule) :-
	H = [P, C],
	removeVarsPrem(P, P1),
	removeVars(C, C1),
	rewriteLogicalRules(T, NewRule1),
	append([[P1, C1]], NewRule1, NewRule).


removeVarsPrem([],[]).
removeVarsPrem([H|T], P) :-
	removeVars(H, P1),
	removeVarsPrem(T, P2),
	append([P1], P2, P).

removeVars([], []).
removeVars([H|T], New) :-
	removeList(['G', 'D'], H, H1),
	removeVars(T, New1),
	append([H1], New1, New).


splitAxiom([],[]).
splitAxiom([X ';' Y], Ax) :-
	splitAxiom([X], Ax1),
	splitAxiom([Y], Ax2),
	append(Ax1, Ax2, Ax).
splitAxiom([X], [X]).

