%%% Filename : p_cutting.pl
%%% Date     : 03.04.2012
%%% Contents : converts an axiom into a logical rule by applying the algorithm
%%%	       step(2): Rule Completion (i.e. Restructuring)
%%% Author   : Lara Katharina Spendier <lara@logic.at>

%% is_rule_completion(+Consequent, -Logical)
%%	creates the logical rule from the given consequent // rule completion
%%	Consequent structure: 
%		[[[Antecedent1],[Succedent1]],[[A2],[S2]],...]
%%	Logical structure:
%%		[[[[PremiseA1], [PremiseS1]],[[PA2], [PS2]],...], [[CA], [CS]]], ...]
is_rule_completion([],[]).

%% logical subformula of the form: o(_ v _), ... (isLogical)
is_rule_completion([[A,S]|T], Logical) :-
	( X = -_; X = '*1'_; X = '*2'_; X = 'o'_
	),
	isLogical(X, _, []),
	( member(X, A) ; member(X, S) ),
%	is_restructuring([A,S], Restructured),
	is_restructuring1([A,S], Restructured, X),
	is_rule_completion(T, Logical1),
	append([Restructured], Logical1, Logical).

%% logical subformula of the form: o1 _, -_,... (isLogical1)
is_rule_completion([[A,S]|T], Logical) :-
	( X = -_; X = '*1'_; X = '*2'_; X = 'o'_
	),
	isLogicalF(X, _, []),
	( member(X, A) ; member(X, S) ),
	is_restructuring1([A,S], Restructured, X),
	is_rule_completion(T, Logical1),
	append([Restructured], Logical1, Logical).

%% logical subformula of the form: o1 _, -_,... (isLogical1)
is_rule_completion([[A,S]|T], Logical) :-
	(X = 'a'; X = 'b'),
	isAtom(X, _, []),
	( member(X, A) ; member(X, S) ),
	is_restructuring1([A,S], Restructured, X),
	is_rule_completion(T, Logical1),
	append([Restructured], Logical1, Logical).

%% is_restructuring(+Consequent, -Restructured)
%% is_restructuring1(+Consequent, -Restructured, +X)
%%	restructures the consequent, i.e.:
%%		- if there is a subformula of the form isLogical, then this
%%		one stays in the consequent and the rest is restructured
%%		- if there are one or several subformulas of the form isLogical1,
%%		then the first occurring subformula stays in the consequent and
%%		the rest is restructured
%%	Consequent 	... consequent of the formula [antecedent, succedent]
%%	Restructured 	... the restructured formula [[Premises], [Consequent]]
%%	X		... the first subformula of the form isLogical1

is_restructuring1([A,S], Restructured, X) :-
	member(X, A),
	is_antecedent_restructuring1(A, RR, X),
	is_succedent_restructuring(S, RR1),
	append(RR1, RR, RR2),	
	is_consequent_restructuring1([A,S], RR3, 1, X),
	append([RR2], [RR3], Restructured).

is_restructuring1([A,S], Restructured, X) :-
	member(X, S),
	is_antecedent_restructuring(A, RR),
	is_succedent_restructuring1(S, RR1, X),
	append(RR1, RR, RR2),	
	is_consequent_restructuring1([A,S], RR3, 2, X),
	append([RR2], [RR3], Restructured).

%% is_antecedent_restructuring(+Antecedent, -RestructuredPremises)
%% is_antecedent_restructuring1(+Antecedent, -RestructuredPremises, +X)
%%	restructures the antecedent, i.e.:
%%		if there is a subformula of the form isLogical, then this
%%		one stays in the consequent. the other subformulas x of the antecedent
%%		go into premises of the form: G => x, D  = [['G'],[x, 'D']]
%%	Antecedent 		... list of subformulas
%%	RestructuredPremises 	... the restructured premises [Antecedent, Succedent]
%%	X			... the first subformula of the form isLogical1

is_antecedent_restructuring([], []).
is_antecedent_restructuring([H|T], RR) :-
	\+ isLogical(H, _, []),
	is_antecedent_restructuring(T, RR1),
	append([H],['D'], H1),
	append([[['G'],H1]], RR1, RR).
is_antecedent_restructuring([_|T], RR) :-
	is_antecedent_restructuring(T, RR).

is_antecedent_restructuring1([], [], _).
is_antecedent_restructuring1([H|T], RR, X) :-
	H \= X,
	is_antecedent_restructuring1(T, RR1, X),
	append([H],['D'], H1),
	append([[['G'],H1]], RR1, RR).
is_antecedent_restructuring1([_|T], RR, X) :-
	is_antecedent_restructuring1(T, RR, X).


%% is_succedent_restructuring(+Succedent, -RestructuredPremises)
%% is_succedent_restructuring1(+Succedent, -RestructuredPremises, +X)
%%	restructures the succedent, i.e.:
%%		if there is a subformula of the form isLogical, then this
%%		one stays in the consequent. the other subformulas x of the succedent
%%		go into premises of the form: G, x => D  = [['G', x],['D']]
%%	Succedent 		... list of subformulas
%%	RestructuredPremises 	... the restructured premises [Antecedent, Succedent]
%%	X			... the first subformula of the form isLogical1
is_succedent_restructuring([], []).
is_succedent_restructuring([H|T], RR) :-
	\+ isLogical(H, _, []),
	is_succedent_restructuring(T, RR1),
	append(['G'], [H], H1),
	append([[H1, ['D']]], RR1, RR).
is_succedent_restructuring([_|T], RR) :-
	is_succedent_restructuring(T, RR).

is_succedent_restructuring1([], [], _).
is_succedent_restructuring1([H|T], RR, X) :-
	H \= X,
	is_succedent_restructuring1(T, RR1, X),
	append(['G'], [H], H1),
	append([[H1, ['D']]], RR1, RR).
is_succedent_restructuring1([_|T], RR, X) :-
	is_succedent_restructuring1(T, RR, X).

%% is_consequent_restructuring(+Consequent, -RestructuredC)
%% is_consequent_restructuring1(+Consequent, -RestructuredC, +Which, +X)
%%	restructures the succedent, i.e.:
%%		if there is a subformula of the form isLogical, then this
%%		one stays in the consequent. the other subformulas x of the succedent
%%		go into premises of the form: G, x => D  = [['G', x],['D']]
%%	Consequent 	... consequent [Antecedent, Succedent]
%%	RestructuredC 	... the restructured consequent [Antecedent, Succedent]
%%	Which		... 1 = X in Antecedent,2 = X in Succedent
%%	X		... the first subformula of the form isLogical1

is_consequent_restructuring([[], S], RR) :-
	length(S, 1),
	RR = [[], S].
is_consequent_restructuring([A, []], RR) :-
	length(A, 1),
	RR = [A, []].
is_consequent_restructuring([A,_], RR) :-
	( X = -_; X = '*1'_; X = '*2'_
	),
	isLogical(X, _, []),
	member(X, A), 
	RR = [['G', X], ['D']].

is_consequent_restructuring([_,S], RR) :-
	( X = -_; X = '*1'_; X = '*2'_
	),
	member(X, S), 
	isLogical(X, _, []),
	RR = [['G'], [X, 'D']].

is_consequent_restructuring1([[], S], RR) :-
	length(S, 1),
	RR = [[], S].
is_consequent_restructuring1([A, []], RR) :-
	length(A, 1),
	RR = [A, []].
is_consequent_restructuring1([A,_], RR, 1, X) :-
	member(X, A), 
	RR = [['G', X], ['D']].
is_consequent_restructuring1([_,S], RR, 2, X) :-
	member(X, S), 
	RR = [['G'], [X, 'D']].
