%%% Filename : p_invertible.pl
%%% Date     : 28.03.2012
%%% Contents : converts an axiom into a logical rule by applying the algorithm
%%%	       step(1): applying the invertible rules
%%% Author   : Lara Katharina Spendier <lara@logic.at>

%% axiom2logical(+Axiom, -Logical)
%%	creates the logical rule from an axiom
%%	Axiom 	... list of axioms provided by the user
%%	Logical ... Logical rule after applying the algorithm
%%	Predefined ...rules (i.e., '*2' is invertible connective)
axiom2logical(Axiom, Logical1, Predefined) :-
	Con = [[[],[Axiom]]],
	is_logical_rule(Con, ConFinal, Predefined),
%	nl,nl,writef('CONFINAL: %q', [ConFinal]),
	removeAxioms(ConFinal, ConFinal1),
%	nl,nl,writef('CONFINAL wo Ax: %q', [ConFinal1]),
	is_rule_completion(ConFinal1, Logical),
%	nl,nl,writef('Logical: %q', [Logical]),
	removeNoSubformulaPremises(Logical, Logical1).
%	nl, nl, writef('Logical NSP: %q', [Logical1]).

%% is_logical_rule(+Axiom, -ConFinal)
%%	starts step (1) for a list of axioms
%% 	Axiom	... list of axioms provided by the user
%%      ConFinal ... [ [Consequent1: [Antecedent],[Succedent] ], ... ] 
is_logical_rule([],[],_).
is_logical_rule([H|T], ConFinal,Predefined) :-
	apply_invertibility([H], ConFinal1,Predefined),
	is_logical_rule(T, ConFinal2,Predefined),
	append(ConFinal1, ConFinal2, ConFinal).

%% removeAxioms(+Rule, -RuleAxiomFree)
%%	removes Rules where we have some axiom (i.e., same variable contained in 
%%	the antecedent and succedent)
removeAxioms([], []).
removeAxioms([[A,S]|T], Rules) :-
	isAxiomRule(A, S, Ax),
	removeAxioms(T, Rules1),
	( Ax = 1 ->
	  Append = []
	; Append = [[A,S]]
	),
	append(Append, Rules1, Rules).

%% isAxiomRule(+Antecedent, +Succedent, -isAxiom)
%%	checks whether a variable of the Antecedent is contained
%%	in the succedent (if yes -> isAxiom = 1)
isAxiomRule([], _, 0).
isAxiomRule([H|T], S, Ax) :-
	( member(H, S) ->
	  Ax = 1
	; isAxiomRule(T, S, Ax)
	).

%% removeNoSubformulaPremises(+Rule, -RuleNoSubformulaPremiseFree)
%%	removes Premises where the atom in there does not have the subformula property
removeNoSubformulaPremises([], []).
removeNoSubformulaPremises([[P,C]|T], LogicalR) :-
	isNoSubformulaPremises(P, C, P1),
	removeNoSubformulaPremises(T, LogicalR1),
	append([[P1,C]], LogicalR1, LogicalR).


%% isNoSubformulaPremises(+Premises, +Conclusion, -PremisesFree)
%%	removes Premises where the atom in there does not have the subformula property
%%	by def. of our grammar, this is never the case when we have both, 'a' and 'b' in the conclusion
%%	if we have '*1'_, '*2'_ in the conclusion, we have to search for such premises
isNoSubformulaPremises(Prem, [AC, SC], Prem) :-
	( X = '*1'('a' v 'b')
        ; X = '*1'('b' v 'a')
        ; X = '*2'('a' v 'b')
        ; X = '*2'('b' v 'a')
        ; X = '*1'('a' & 'b')
        ; X = '*1'('b' & 'a')
        ; X = '*2'('a' & 'b')
        ; X = '*2'('b' & 'a')
        ; X = '*1'('a' -> 'b')
        ; X = '*1'('b' -> 'a')
        ; X = '*2'('a' -> 'b')
        ; X = '*2'('b' -> 'a')),
	(member(X, AC) ; member(X, SC)).

isNoSubformulaPremises(Prem, [AC,SC], PremR) :-
	( X = '*1' 'a'; X = '*2' 'a'; X = '*1'('*1' 'a'); X = '*1'('*2' 'a'); X = '*2'('*1' 'a'); X = '*2'('*2' 'a') ),
	(member(X,AC) ; member(X, SC)),
	removeNoSubFormula('a', Prem, PremR).

isNoSubformulaPremises(Prem, [AC,SC], PremR) :-
	( X = '*1' 'b'; X = '*2' 'b'; X = '*1'('*1' 'b'); X = '*1'('*2' 'b'); X = '*2'('*1' 'b'); X = '*2'('*2' 'b') ),
	(member(X,AC) ; member(X, SC)),
	removeNoSubFormula('b', Prem, PremR).


%% apply_invertibility(+Axiom, -ConFinal)
%%	starts application of invertibility to the axiom
%%	checks if the axiom is invertible (is_invertible) and applies either
%%	invertible-right-rules or invertible-left-rules to the list
%%	also considers that some invertible rules create 2 different rules
%% 	Axiom	... list of axioms provided by the user
%%      ConFinal ... [Antecedent,Succedent]
apply_invertibility([],[],_).

apply_invertibility([[A,S]|T], ConFinal,Predefined) :-
	is_invertible(S,Predefined),
	is_invertible_right([A,S], F,Predefined),
	append(F, T, CF1),
	apply_invertibility(CF1, ConFinal,Predefined).

apply_invertibility([[A,S]|T], ConFinal,Predefined) :-
	is_invertible(A,Predefined),
	is_invertible_left([A,S], F,Predefined),
	append(F, T, CF1),
	apply_invertibility(CF1, ConFinal,Predefined).

apply_invertibility([[A,S]|T], ConFinal,Predefined) :-
	\+ is_invertible(A,Predefined),
	\+ is_invertible(S,Predefined),
	apply_invertibility(T, CF1,Predefined),
	append([[A,S]], CF1, ConFinal).

%% is_invertible(+Formula)
%%	checks if the given formula is invertible (-> defined connectives)
%%	returns false if the given formula is not invertible
%%	Formula	... the formula which could be invertible
is_invertible([_ -> _|_],_).
is_invertible([_ & _|_],_).
is_invertible([_ v _|_],_).
is_invertible(['*2' _|_],1).
%is_invertible(['*1' _|_]).
%is_invertible(['*2' _|_]).

is_invertible([_|T],Predefined) :-
	is_invertible(T,Predefined).

%% is_invertible_right(+Consequent, -Final),
%% 	applies the right invertible rules (->,r),(v,r),(&,r) and (o,r)
%%	Consequent	... given consequent of the form [Antecedent, Succedent]
%%	Final		... list of consequents of the form [[A,S],[A,S],...]
%%			    see e.g. (&,r) which creates two rules!
%% (->, r): => A -> B .. A => B
is_invertible_right([A, S], F,_) :-
	member(Ax1 -> Ax2, S),
	remove(Ax1 -> Ax2, S, S1),
	append([Ax2], S1, S2),
	append([Ax1], A, A1),
	F = [[A1, S2]].

%% (v, r):  => A v B  .. => A, B
is_invertible_right([A, S], F,_) :-
	member(Ax1 v Ax2, S),
	remove(Ax1 v Ax2, S, S1), 
	append([Ax1, Ax2], S1, S2),
	F = [[A, S2]].

%% (&, r):  => A & B  .. R1: => A, R2: => B  % generates 2 rules!
is_invertible_right([A,S], F,_) :-
	member(Ax1 & Ax2, S),
	remove(Ax1 & Ax2, S, S1),
	append([Ax1], S1, S2),
	append([Ax2], S1, S3),
	F = [[A, S2], [A, S3]].

%% (o,r): => oA .. R: A, -A =>
is_invertible_right([A, S], F,1) :-
	member('*2' Ax, S),
	remove('*2' Ax, S, S1),
	append([Ax, '*1'Ax], A, A1),
	F = [[A1, S1]].


%% is_invertible_left(+Consequent, -Final),
%% 	applies the left invertible rules (->,l),(v,l),(&,l) and (o,l)
%%	Consequent	... given consequent of the form [Antecedent, Succedent]
%%	Final		... list of consequents of the form [[A,S],[A,S],...]
%%			    see e.g. (v,l) which creates two rules!
%% (->, l): A -> B => .. R1: => A, R2: B =>
is_invertible_left([A,S], F,_) :-
	member(Ax1 -> Ax2, A), 
	remove(Ax1 -> Ax2, A, A1),
	append([Ax1], S, S1),
	append([Ax2], A1, A2),
	F = [[A1, S1], [A2, S]].

%% (v, l):  A v B =>  .. R1: A =>, R2: B =>
is_invertible_left([A,S], F, _) :-
	member(Ax1 v Ax2, A),
	remove(Ax1 v Ax2, A, A1),
	append([Ax1], A1, A2),
	append([Ax2], A1, A3),
	F = [[A2, S], [A3, S]].

%% (&, l):  A & B =>  .. R1: => A, R2: => B
is_invertible_left([A, S], F, _) :-
	member(Ax1 & Ax2, A),
	remove(Ax1 & Ax2, A, A1), 
	append([Ax1, Ax2], A1, A2),
	F = [[A2, S]].

%% (o, l): oA => .. R1: => A, R2: => -A
is_invertible_left([A,S], F, 1) :-
	member('*2' Ax, A), 
	remove('*2' Ax, A, A1), 
	append([Ax], S, S1),
	append(['*1'Ax], S, S2),
	F = [[A1, S1], [A1, S2]].
